@@ -38,18 +38,14 @@ test_that("invalid `solution_terms` warns or fails", {
       info = tstsetup
     )
 
-    # Invalid length:
-    expect_error(
-      do.call(project, c(
-        list(object = refmods[[args_prj_i$tstsetup_ref]],
-             solution_terms = rep_len(args_prj_i$solution_terms,
-                                      length.out = 1e4)),
-        excl_nonargs(args_prj_i, nms_excl_add = "solution_terms")
-      )),
-      paste("^Argument 'solution_terms' contains more terms than the number of",
-            "terms in the reference model\\.$"),
-      info = tstsetup
-    )
+    # Repeating solution terms:
+    p_long <- do.call(project, c(
+      list(object = refmods[[args_prj_i$tstsetup_ref]],
+           solution_terms = rep_len(args_prj_i$solution_terms,
+                                    length.out = 1e4)),
+      excl_nonargs(args_prj_i, nms_excl_add = "solution_terms")
+    ))
+    expect_identical(p_long, prjs[[tstsetup]])
 
     # Invalid type:
     for (solterms_crr in list(2, 1:3, list(solterms_x, solterms_x))) {
@@ -81,8 +77,8 @@ test_that("invalid `solution_terms` warns or fails", {
                solution_terms = solterms_crr),
           excl_nonargs(args_prj_i, nms_excl_add = "solution_terms")
         )),
-        paste("At least one element of `solution_terms` could not be found",
-              "in the table of solution terms"),
+        paste("The following element\\(s\\) of `solution_terms` could not be",
+              "found in the table of possible solution terms"),
         info = tstsetup_crr
       )
       projection_tester(p,
@@ -97,13 +93,14 @@ test_that("invalid `solution_terms` warns or fails", {
 
 test_that("`object` of class \"stanreg\" or \"brmsfit\" works", {
   skip_if_not(run_prj)
-  tstsetups <- grep("\\.glm\\.gauss.*\\.solterms_x\\.clust$", names(prjs),
+  tstsetups <- grep("\\.brnll\\..*\\.solterms_x\\.clust$", names(prjs),
                     value = TRUE)
   for (tstsetup in tstsetups) {
     args_prj_i <- args_prj[[tstsetup]]
     p_fit <- do.call(project, c(
       list(object = fits[[args_prj_i$tstsetup_fit]]),
-      excl_nonargs(args_prj_i)
+      excl_nonargs(args_prj_i),
+      excl_nonargs(args_ref[[args_prj_i$tstsetup_ref]])
     ))
     expect_identical(p_fit, prjs[[tstsetup]], ignore.environment = TRUE,
                      info = tstsetup)
@@ -122,7 +119,7 @@ test_that(paste(
     fam_crr <- args_vs[[tstsetup_vs]]$fam_nm
     nterms_crr <- args_prj_vs[[tstsetup]]$nterms
     if (is.null(nterms_crr)) {
-      nterms_crr <- vss[[tstsetup_vs]]$suggested_size
+      nterms_crr <- suggest_size(vss[[tstsetup_vs]], warnings = FALSE)
     }
     if (length(nterms_crr) == 1) {
       solterms_expected_crr <- vss[[tstsetup_vs]]$solution_terms[
@@ -148,12 +145,45 @@ test_that(paste(
         value = TRUE
       )
       match_prj <- sapply(tstsetup_tries, function(tstsetup_try) {
-        setequal(solterms_expected_crr, prjs[[tstsetup_try]]$solution_terms)
+        setequal(solterms_expected_crr, prjs[[tstsetup_try]]$solution_terms) &&
+          args_prj_vs[[tstsetup]]$prj_nm == args_prj[[tstsetup_try]]$prj_nm
       })
       tstsetup_match_prj <- tstsetup_tries[match_prj]
       if (length(tstsetup_match_prj) == 1) {
-        expect_identical(prjs_vs[[tstsetup]], prjs[[tstsetup_match_prj]],
-                         ignore.environment = TRUE, info = tstsetup)
+        if (identical(prjs_vs[[tstsetup]]$solution_terms,
+                      prjs[[tstsetup_match_prj]]$solution_terms)) {
+          expect_identical(prjs_vs[[tstsetup]], prjs[[tstsetup_match_prj]],
+                           ignore.environment = TRUE, info = tstsetup)
+        } else {
+          expect_setequal(prjs_vs[[tstsetup]]$solution_terms,
+                          prjs[[tstsetup_match_prj]]$solution_terms)
+          expect_equal(
+            lapply(seq_along(prjs_vs[[tstsetup]]$outdmin), function(s_idx) {
+              outdmin_s <- prjs_vs[[tstsetup]]$outdmin[[s_idx]]
+              outdmin_s$beta <- outdmin_s$beta[
+                rownames(prjs[[tstsetup_match_prj]]$outdmin[[s_idx]]$beta), ,
+                drop = FALSE
+              ]
+              outdmin_s$formula <- prjs[[tstsetup_match_prj]]$outdmin[[s_idx]][[
+                "formula"
+              ]]
+              outdmin_s$x <- outdmin_s$x[
+                , colnames(prjs[[tstsetup_match_prj]]$outdmin[[s_idx]]$x),
+                drop = FALSE
+              ]
+              return(outdmin_s)
+            }),
+            prjs[[tstsetup_match_prj]]$outdmin,
+            tolerance = 1e1 * .Machine$double.eps, info = tstsetup
+          )
+          prj_nms <- names(prjs_vs[[tstsetup]])
+          expect_identical(prj_nms, names(prjs[[tstsetup_match_prj]]),
+                           info = tstsetup)
+          prj_el_excl <- !prj_nms %in% c("solution_terms", "outdmin")
+          expect_equal(prjs_vs[[tstsetup]][prj_el_excl],
+                       prjs[[tstsetup_match_prj]][prj_el_excl],
+                       tolerance = .Machine$double.eps, info = tstsetup)
+        }
       } else if (length(tstsetup_match_prj) > 1) {
         stop("Unexpected number of matches.")
       }
@@ -166,7 +196,7 @@ test_that(paste(
         refmod_expected = refmods[[args_prj_vs[[tstsetup]]$tstsetup_ref]],
         nprjdraws_expected = args_prj_vs[[tstsetup]]$nclusters,
         p_type_expected = TRUE,
-        prjdraw_weights_expected = prjs_vs[[tstsetup]][[1]]$weights
+        prjdraw_weights_expected = prjs_vs[[tstsetup]][[1]]$wdraws_prj
       )
     }
   }
@@ -184,7 +214,7 @@ test_that(paste(
     fam_crr <- args_cvvs[[tstsetup_cvvs]]$fam_nm
     nterms_crr <- args_prj_cvvs[[tstsetup]]$nterms
     if (is.null(nterms_crr)) {
-      nterms_crr <- cvvss[[tstsetup_cvvs]]$suggested_size
+      nterms_crr <- suggest_size(cvvss[[tstsetup_cvvs]], warnings = FALSE)
     }
     if (length(nterms_crr) == 1) {
       solterms_expected_crr <- cvvss[[tstsetup_cvvs]]$solution_terms[
@@ -210,12 +240,45 @@ test_that(paste(
         value = TRUE
       )
       match_prj <- sapply(tstsetup_tries, function(tstsetup_try) {
-        setequal(solterms_expected_crr, prjs[[tstsetup_try]]$solution_terms)
+        setequal(solterms_expected_crr, prjs[[tstsetup_try]]$solution_terms) &&
+          args_prj_cvvs[[tstsetup]]$prj_nm == args_prj[[tstsetup_try]]$prj_nm
       })
       tstsetup_match_prj <- tstsetup_tries[match_prj]
       if (length(tstsetup_match_prj) == 1) {
-        expect_identical(prjs_cvvs[[tstsetup]], prjs[[tstsetup_match_prj]],
-                         ignore.environment = TRUE, info = tstsetup)
+        if (identical(prjs_cvvs[[tstsetup]]$solution_terms,
+                      prjs[[tstsetup_match_prj]]$solution_terms)) {
+          expect_identical(prjs_cvvs[[tstsetup]], prjs[[tstsetup_match_prj]],
+                           ignore.environment = TRUE, info = tstsetup)
+        } else {
+          expect_setequal(prjs_cvvs[[tstsetup]]$solution_terms,
+                          prjs[[tstsetup_match_prj]]$solution_terms)
+          expect_equal(
+            lapply(seq_along(prjs_cvvs[[tstsetup]]$outdmin), function(s_idx) {
+              outdmin_s <- prjs_cvvs[[tstsetup]]$outdmin[[s_idx]]
+              outdmin_s$beta <- outdmin_s$beta[
+                rownames(prjs[[tstsetup_match_prj]]$outdmin[[s_idx]]$beta), ,
+                drop = FALSE
+              ]
+              outdmin_s$formula <- prjs[[tstsetup_match_prj]]$outdmin[[s_idx]][[
+                "formula"
+              ]]
+              outdmin_s$x <- outdmin_s$x[
+                , colnames(prjs[[tstsetup_match_prj]]$outdmin[[s_idx]]$x),
+                drop = FALSE
+              ]
+              return(outdmin_s)
+            }),
+            prjs[[tstsetup_match_prj]]$outdmin,
+            tolerance = 1e1 * .Machine$double.eps, info = tstsetup
+          )
+          prj_nms <- names(prjs_cvvs[[tstsetup]])
+          expect_identical(prj_nms, names(prjs[[tstsetup_match_prj]]),
+                           info = tstsetup)
+          prj_el_excl <- !prj_nms %in% c("solution_terms", "outdmin")
+          expect_equal(prjs_cvvs[[tstsetup]][prj_el_excl],
+                       prjs[[tstsetup_match_prj]][prj_el_excl],
+                       tolerance = .Machine$double.eps, info = tstsetup)
+        }
       } else if (length(tstsetup_match_prj) > 1) {
         stop("Unexpected number of matches.")
       }
@@ -228,7 +291,7 @@ test_that(paste(
         refmod_expected = refmods[[args_prj_cvvs[[tstsetup]]$tstsetup_ref]],
         nprjdraws_expected = args_prj_cvvs[[tstsetup]]$nclusters,
         p_type_expected = TRUE,
-        prjdraw_weights_expected = prjs_cvvs[[tstsetup]][[1]]$weights
+        prjdraw_weights_expected = prjs_cvvs[[tstsetup]][[1]]$wdraws_prj
       )
     }
   }
@@ -277,10 +340,61 @@ test_that("non-clustered projection does not require a seed", {
     args_prj_i <- args_prj[[tstsetup]]
     p_orig <- prjs[[tstsetup]]
     rand_new1 <- runif(1) # Just to advance `.Random.seed[2]`.
-    p_new <- do.call(project, c(
-      list(object = refmods[[args_prj_i$tstsetup_ref]]),
-      excl_nonargs(args_prj_i, nms_excl_add = "seed")
-    ))
+    if (args_prj_i$prj_nm == "augdat" && args_prj_i$fam_nm == "cumul" &&
+        !any(grepl("\\|", args_prj_i$solution_terms))) {
+      warn_expected <- "non-integer #successes in a binomial glm!"
+    } else if (!is.null(args_prj_i$avoid.increase) &&
+               any(grepl("\\|", args_prj_i$solution_terms))) {
+      warn_expected <- warn_mclogit
+    } else {
+      warn_expected <- NA
+    }
+    expect_warning(
+      p_new <- do.call(project, c(
+        list(object = refmods[[args_prj_i$tstsetup_ref]]),
+        excl_nonargs(args_prj_i, nms_excl_add = "seed")
+      )),
+      warn_expected
+    )
+    if (args_prj_i$mod_nm %in% c("glmm", "gamm") &&
+        any(grepl("\\|", args_prj_i$solution_terms))) {
+      if (getOption("projpred.mlvl_pred_new", FALSE)) {
+        # In this case, the multilevel submodel fitters (fit_glmer_callback(),
+        # fit_gamm_callback(), fit_cumul_mlvl(), fit_categ_mlvl()) should still
+        # be deterministic, but the prediction from the fitted submodels is not
+        # (because of the group-level effects drawn randomly by repair_re() (for
+        # all group levels; here, only the existing ones are relevant)). Thus,
+        # we cannot test the whole project() output, but need to restrict
+        # ourselves to the output of as.matrix.projection().
+        if (args_prj_i$mod_nm == "gamm") {
+          # Skipping GAMMs because of issue #131.
+          # TODO (GAMMs): Fix this.
+          next
+        }
+        prjmat_orig <- as.matrix(p_orig)
+        prjmat_new <- as.matrix(p_new)
+        if (args_prj_i$fam_nm == "gauss" || args_prj_i$prj_nm == "latent") {
+          # The projected dispersion parameter is affected by the group-level
+          # effects drawn randomly by repair_re() (for all group levels):
+          prjmat_new[, "sigma"] <- prjmat_orig[, "sigma"]
+        }
+        expect_equal(prjmat_new, prjmat_orig, info = tstsetup,
+                     tolerance = .Machine$double.eps)
+        # To facilitate the `if` conditions here:
+        p_new <- p_orig
+      } else if (args_prj_i$prj_nm == "augdat" &&
+                 args_prj_i$fam_nm == "cumul" && args_prj_i$mod_nm == "glmm") {
+        for (idx_s in seq_along(p_new$outdmin)) {
+          if (!is.null(p_new$outdmin[[idx_s]][["L"]])) {
+            # We could also use `"sparseMatrix"` instead of `"Matrix"`:
+            expect_equal(as(p_new$outdmin[[idx_s]][["L"]], "Matrix"),
+                         as(p_orig$outdmin[[idx_s]][["L"]], "Matrix"),
+                         info = tstsetup)
+            p_new$outdmin[[idx_s]][["L"]] <- p_orig$outdmin[[idx_s]][["L"]]
+          }
+        }
+      }
+    }
     expect_equal(p_new, p_orig, info = tstsetup)
   }
 })
@@ -327,6 +441,7 @@ test_that("for GLMs, `regul` has an expected effect", {
   stopifnot(regul_tst[1] == regul_default)
   stopifnot(all(diff(regul_tst) > 0))
   tstsetups <- grep("\\.glm\\..*\\.clust$", names(prjs), value = TRUE)
+  tstsetups <- grep(fam_nms_aug_regex, tstsetups, value = TRUE, invert = TRUE)
   for (tstsetup in tstsetups) {
     args_prj_i <- args_prj[[tstsetup]]
     ndr_ncl <- ndr_ncl_dtls(args_prj_i)
